@@ -274,20 +274,14 @@ class Assets(WMLResource):
                                     )
                         if put_response.status_code == 201 or put_response.status_code == 200:
                             # Step4: Complete attachment
-                            if not self._ICP:
-                                complete_response = requests.post(
-                                    # self._client.service_instance._href_definitions.get_attachment_href(asset_id,attachment_id)+"/complete",
-                                    self._client.service_instance._href_definitions.get_attachment_complete_href(asset_id, attachment_id),
-                                    headers=self._client._get_headers(),
-                                    params = self._client._params()
 
-                                )
-                            else:
-                                complete_response = requests.post(
-                                    self._client.service_instance._href_definitions.get_attachment_href(asset_id,attachment_id)+"/complete",
-                                    headers=self._client._get_headers(),
-                                    params=self._client._params()
-                                )
+                            complete_response = requests.post(
+                                self._client.service_instance._href_definitions.get_attachment_complete_href(asset_id, attachment_id),
+                                headers=self._client._get_headers(),
+                                params = self._client._params()
+
+                            )
+
                             if complete_response.status_code == 200:
                                 print(Messages.get_message(message_id="success"))
                                 return self._get_required_element_from_response(asset_details)
@@ -623,12 +617,9 @@ class Assets(WMLResource):
     #
     #     Assets._validate_type(asset_uid, u'asset_uid', STR_TYPE, True)
     #
-    #     if not self._ICP:
-    #         response = requests.get(self._client.service_instance._href_definitions.get_data_asset_href(asset_uid), params=self._client._params(),
-    #                                 headers=self._client._get_headers())
-    #     else:
-    #         response = requests.get(self._client.service_instance._href_definitions.get_data_asset_href(asset_uid), params=self._client._params(),
-    #                                 headers=self._client._get_headers())
+    #     response = requests.get(self._client.service_instance._href_definitions.get_data_asset_href(asset_uid), params=self._client._params(),
+    #                             headers=self._client._get_headers())
+    #
     #     if response.status_code == 200:
     #         return response.json()["href"]
     #     else:
