@@ -16,13 +16,15 @@ from __future__ import print_function
 import ibm_watson_machine_learning._wrappers.requests as requests
 
 from ibm_watson_machine_learning.messages.messages import Messages
-from ibm_watson_machine_learning.utils import SPACES_DETAILS_TYPE, INSTANCE_DETAILS_TYPE, MEMBER_DETAILS_TYPE,DATA_ASSETS_DETAILS_TYPE, STR_TYPE, STR_TYPE_NAME, docstring_parameter, meta_props_str_conv, str_type_conv, get_file_from_cos
+from ibm_watson_machine_learning.utils import SPACES_DETAILS_TYPE, INSTANCE_DETAILS_TYPE, MEMBER_DETAILS_TYPE, \
+    DATA_ASSETS_DETAILS_TYPE, STR_TYPE, STR_TYPE_NAME, docstring_parameter, meta_props_str_conv, str_type_conv, \
+    get_file_from_cos
 from ibm_watson_machine_learning.metanames import ConnectionMetaNames
 from ibm_watson_machine_learning.wml_resource import WMLResource
 from ibm_watson_machine_learning.wml_client_error import WMLClientError, ApiRequestFailure
 import os
 import json
-from urllib.parse import unquote
+from urllib.parse import unquote, quote
 
 _DEFAULT_LIST_LENGTH = 0
 
@@ -106,9 +108,10 @@ class Connections(WMLResource):
             header_param = self._client._get_headers()
 
         if connection_id:
-            response = requests.get(self._client.service_instance._href_definitions.get_connection_by_id_href(connection_id),
-                                    params=self._client._params(),
-                                    headers=header_param)
+            response = requests.get(
+                self._client.service_instance._href_definitions.get_connection_by_id_href(connection_id),
+                params=self._client._params(),
+                headers=header_param)
 
             if response.status_code == 200:
                 return self._get_required_element_from_response(
@@ -386,10 +389,11 @@ class Connections(WMLResource):
 
         datasource_details = self._handle_response(200, u'list datasource types', response)['resources']
         space_values = [
-            (m[u'entity'][u'name'], m[u'metadata'][u'asset_id'],m['metadata']['create_time'], m[u'entity'][u'datasource_type']) for
+            (m[u'entity'][u'name'], m[u'metadata'][u'asset_id'], m['metadata']['create_time'],
+             m[u'entity'][u'datasource_type']) for
             m in datasource_details]
 
-        self._list(space_values, [u'NAME', u'ID', u'CREATED',  u'DATASOURCE_TYPE_ID', ], None, None)
+        self._list(space_values, [u'NAME', u'ID', u'CREATED', u'DATASOURCE_TYPE_ID', ], None, None)
 
     @docstring_parameter({'str_type': STR_TYPE_NAME})
     def list_uploaded_db_drivers(self):
@@ -421,7 +425,6 @@ class Connections(WMLResource):
 
         self._list(jars, [u'NAME'], None, None)
 
-
     @docstring_parameter({'str_type': STR_TYPE_NAME})
     def get_datasource_type_uid_by_name(self, name):
         """
@@ -451,8 +454,10 @@ class Connections(WMLResource):
         else:
             header_param = self._client._get_headers()
 
-        response = requests.get(self._client.service_instance._href_definitions.get_connection_data_types_href(),
-                                headers=header_param)
+        sess = self.requests_retry_session()
+        response = sess.get(self._client.service_instance._href_definitions.get_connection_data_types_href(),
+                            headers=header_param)
+
         datasource_id = 'None'
         datasource_details = self._handle_response(200, u'list datasource types', response)['resources']
         for i, ds_resource in enumerate(datasource_details):
@@ -463,8 +468,8 @@ class Connections(WMLResource):
     @docstring_parameter({'str_type': STR_TYPE_NAME})
     def upload_db_driver(self, path: str):
         """
-           List uploaded db driver jars.
-           Supported for IBM Cloud Pak for Data only.
+           Upload db driver jar.
+           Supported for IBM Cloud Pak for Data only, version 4.0.4 and above.
 
            **Parameters**
 
@@ -480,20 +485,16 @@ class Connections(WMLResource):
         if not self._client.ICP:
             raise WMLClientError('Not supported on this environment.')
 
-        import urllib
-        import requests
-
         driver_file_name = path.split('/')[-1]
 
         with open(path, 'rb') as fdata:
             content_upload_url = self._client.service_instance._href_definitions.get_wsd_model_attachment_href() + \
-                                 "dbdrivers/" + urllib.parse.quote(driver_file_name, safe='')
+                                 "dbdrivers/" + quote(driver_file_name, safe='')
             response = requests.put(
                 content_upload_url,
                 files={'file': ('native', fdata, 'application/octet-stream', {'Expires': '0'})},
                 headers=self._client._get_headers(no_content_type=True),
-                params=self._client._params(),
-                verify=False
+                params=self._client._params()
             )
 
             self._client.repository._handle_response(201, 'uploading db driver jar', response)
@@ -502,7 +503,7 @@ class Connections(WMLResource):
     def sign_db_driver_url(self, jar_name: str):
         """
            Get signed db driver jar url to be used during creating of JDBC generic connection. The jar name passed as argument needs to be uploaded into system first.
-           Supported for IBM Cloud Pak for Data only.
+           Supported for IBM Cloud Pak for Data only, version 4.0.4 and above..
 
            **Parameters**
 
@@ -518,11 +519,8 @@ class Connections(WMLResource):
         if not self._client.ICP:
             raise WMLClientError('Not supported on this environment.')
 
-        import urllib
-        import requests
-
         signed_url = self._client.service_instance._href_definitions.get_wsd_model_attachment_href() + \
-                     urllib.parse.quote("dbdrivers/" + jar_name, safe='') + "/signed"
+                     quote("dbdrivers/" + jar_name, safe='') + "/signed"
         params = self._client._params()
 
         params['expires_in'] = 5000
@@ -530,11 +528,28 @@ class Connections(WMLResource):
         response = requests.post(
             signed_url,
             headers=self._client._get_headers(no_content_type=True),
-            params=params,
-            verify=False
+            params=params
         )
 
         self._client.repository._handle_response(201, 'signing db driver url', response, json_response=False)
 
         return unquote(response.headers['Location'])
 
+    @staticmethod
+    def requests_retry_session(retries=3, backoff_factor=0.3, status_forcelist=(500, 502, 503, 504), session=None):
+        from requests.adapters import HTTPAdapter
+        from requests.packages.urllib3.util.retry import Retry
+
+        session = session or requests.Session()
+        retry = Retry(
+            total=retries,
+            read=retries,
+            connect=retries,
+            backoff_factor=backoff_factor,
+            status_forcelist=status_forcelist,
+            raise_on_status=False
+        )
+        adapter = HTTPAdapter(max_retries=retry)
+        session.mount('http://', adapter)
+        session.mount('https://', adapter)
+        return session
