@@ -195,30 +195,18 @@ class Deployments(WMLResource):
                                 "It is mandatory is set the space. Use client.set.default_space(<SPACE_GUID>) to set the space.")
 
             url = self._client.service_instance._href_definitions.get_deployments_href()
-            if not self._ICP:
-                if self._client.CLOUD_PLATFORM_SPACES:
-                    response = requests.post(
-                        url,
-                        json=metaProps,
-                        params=self._client._params(),  # version is mandatory
-                        headers=self._client._get_headers())
-                else:
-                    response = requests.post(
-                        url,
-                        json=metaProps,
-                        headers=self._client._get_headers())
+
+            if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
+                response = requests.post(
+                    url,
+                    json=metaProps,
+                    params=self._client._params(),  # version is mandatory
+                    headers=self._client._get_headers())
             else:
-                if self._client.ICP_PLATFORM_SPACES:
-                    response = requests.post(
-                        url,
-                        json=metaProps,
-                        params=self._client._params(),  # version is mandatory
-                        headers=self._client._get_headers())
-                else:
-                    response = requests.post(
-                        url,
-                        json=metaProps,
-                        headers=self._client._get_headers())
+                response = requests.post(
+                    url,
+                    json=metaProps,
+                    headers=self._client._get_headers())
 
             ## Post Deployment call executed
             if response.status_code == 202:
@@ -632,23 +620,17 @@ class Deployments(WMLResource):
 
         deployment_url = self._client.service_instance._href_definitions.get_deployment_href(deployment_uid)
 
-        if not self._ICP:
+        if not self._ICP or self._client.ICP_PLATFORM_SPACES:
             response_delete = requests.delete(
                 deployment_url,
                 params=self._client._params(),
                 headers=self._client._get_headers())
         else:
-            if self._client.ICP_PLATFORM_SPACES:
-                response_delete = requests.delete(
-                    deployment_url,
-                    params=self._client._params(),
-                    headers=self._client._get_headers())
-            else:
-                response_delete = requests.delete(
-                    deployment_url,
-                    #TODO:  The below line needs to uncommeted after /v4/deployment accespts query param
-                   # params=self._client._params(),
-                    headers=self._client._get_headers())
+            response_delete = requests.delete(
+                deployment_url,
+                #TODO:  The below line needs to uncommeted after /v4/deployment accespts query param
+               # params=self._client._params(),
+                headers=self._client._get_headers())
 
         return self._handle_response(204, u'deployment deletion', response_delete, False)
 
@@ -754,34 +736,19 @@ class Deployments(WMLResource):
         else:
             scoring_url = self._wml_credentials["url"] + "/v4/deployments/" + deployment_id + "/predictions"
 
-        if not self._ICP:
-            if self._client.CLOUD_PLATFORM_SPACES:
-                params = self._client._params()
-                del params[u'space_id']
-                response_scoring = requests.post(
-                    scoring_url,
-                    json=payload,
-                    params=params,  # version parameter is mandatory
-                    headers=headers)
-            else:
-                response_scoring = requests.post(
-                    scoring_url,
-                    json=payload,
-                    headers=headers)
+        if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
+            params = self._client._params()
+            del params[u'space_id']
+            response_scoring = requests.post(
+                scoring_url,
+                json=payload,
+                params=params,  # version parameter is mandatory
+                headers=headers)
         else:
-            if self._client.ICP_PLATFORM_SPACES:
-                params = self._client._params()
-                del params[u'space_id']
-                response_scoring = requests.post(
-                    scoring_url,
-                    json=payload,
-                    params=params,  # version parameter is mandatory
-                    headers=headers)
-            else:
-                response_scoring = requests.post(
-                    scoring_url,
-                    json=payload,
-                    headers=headers)
+            response_scoring = requests.post(
+                scoring_url,
+                json=payload,
+                headers=headers)
 
         return self._handle_response(200, u'scoring', response_scoring)
 
@@ -1093,10 +1060,7 @@ class Deployments(WMLResource):
 
         url = self._client.service_instance._href_definitions.get_deployment_href(deployment_uid)
 
-        if not self._ICP:
-            response = requests.patch(url, json=patch_payload, params=self._client._params(), headers=self._client._get_headers())
-        else:
-            response = requests.patch(url, json=patch_payload, params=self._client._params(), headers=self._client._get_headers())
+        response = requests.patch(url, json=patch_payload, params=self._client._params(), headers=self._client._get_headers())
 
         if patch_job and response.status_code == 202:
             updated_details = self._handle_response(202, u'deployment asset patch', response)
@@ -1575,14 +1539,8 @@ class Deployments(WMLResource):
     #     url = self._client.service_instance._href_definitions.get_envs_href()
     #     metadata = self.EnvironmentMetanames._generate_resource_metadata(meta_props)
     #
-    #     if not self._ICP:
-    #         response_post = requests.post(url, json=metadata,
-    #                                           headers=self._client._get_headers())
-    #
-    #
-    #     else:
-    #         response_post = requests.post(url, json=metadata,
-    #                                       headers=self._client._get_headers())
+    #     response_post = requests.post(url, json=metadata,
+    #                                     headers=self._client._get_headers())
     #
     #     return self._handle_response(201, u'deployment job deletion', response_post, False)
     #
@@ -1620,15 +1578,9 @@ class Deployments(WMLResource):
     #
     #     url = self._client.service_instance._href_definitions.get_env_href(env_id)
     #
-    #     if not self._ICP:
-    #         response_delete = requests.delete(
-    #             url,
-    #             headers=self._client._get_headers())
-    #     else:
-    #         response_delete = requests.delete(
-    #             url,
-    #             headers=self._client._get_headers(),
-    #             verify=False)
+    #     response_delete = requests.delete(
+    #           url,
+    #           headers=self._client._get_headers())
     #
     #     return self._handle_response(204, u'environment deletion', response_delete, False)
     #
