@@ -25,7 +25,7 @@ from ibm_watson_machine_learning.utils.autoai.errors import LocalInstanceButRemo
     TSNotSupported, ParamOutOfRange, ImputationListNotSupported
 from ibm_watson_machine_learning.utils.autoai.utils import check_dependencies_versions, \
     validate_additional_params_for_optimizer, validate_optimizer_enum_values, \
-    translate_imputation_string_strategy_to_enum
+    translate_imputation_string_strategy_to_enum, translate_estimator_string_to_enum
 from ibm_watson_machine_learning.workspace import WorkSpace
 from .engines import WMLEngine
 from .optimizers import LocalAutoPipelines, RemoteAutoPipelines
@@ -187,6 +187,8 @@ class AutoAI(BaseExperiment):
                   numerical_imputation_value: float = None,
                   imputation_threshold: float = None,
                   retrain_on_holdout: int = True,
+                  categorical_columns: list = None,
+                  numerical_columns: list = None,
                   **kwargs) -> Union['RemoteAutoPipelines', 'LocalAutoPipelines']:
         """
         Initialize an AutoAi optimizer.
@@ -352,6 +354,12 @@ class AutoAI(BaseExperiment):
         retrain_on_holdout: bool, optional
             If True final pipelines will be train also on holdout data.
 
+        categorical_columns: list, optional
+            List of columns names that must be treated as categorical.
+
+        numerical_columns: list, optional
+            List of columns names that must be treated as numerical.
+
         Returns
         -------
         RemoteAutoPipelines or LocalAutoPipelines, depends on how you initialize the AutoAI object.
@@ -439,8 +447,12 @@ class AutoAI(BaseExperiment):
             else:
                 return x
 
+        def translate_str_include_only_estimators_param(x):
+            return [translate_estimator_string_to_enum(estimator) for estimator in x]
+
         categorical_imputation_strategy = translate_str_imputation_param(categorical_imputation_strategy)
         numerical_imputation_strategy = translate_str_imputation_param(numerical_imputation_strategy)
+        include_only_estimators = translate_str_include_only_estimators_param(include_only_estimators) if include_only_estimators else None
 
         validate_optimizer_enum_values(
             prediction_type=prediction_type,
@@ -556,6 +568,8 @@ class AutoAI(BaseExperiment):
                 numerical_imputation_value=numerical_imputation_value,
                 imputation_threshold=imputation_threshold,
                 retrain_on_holdout=retrain_on_holdout,
+                numerical_columns=numerical_columns,
+                categorical_columns=categorical_columns,
                 **reduced_kwargs
             )
             optimizer._workspace = self._workspace
