@@ -23,7 +23,7 @@ from numpy import ndarray
 from pandas import DataFrame
 
 from ibm_watson_machine_learning.helpers.connections import (
-    DataConnection, S3Location, FSLocation, AssetLocation, WSDAssetLocation, ContainerLocation)
+    DataConnection, S3Location, FSLocation, AssetLocation, WSDAssetLocation, ContainerLocation, DatabaseLocation)
 from ibm_watson_machine_learning.preprocessing import DataJoinPipeline
 from ibm_watson_machine_learning.utils.autoai.enums import (
     RunStateTypes, PipelineTypes, TShirtSize, ClassificationAlgorithms, RegressionAlgorithms, DataConnectionTypes)
@@ -343,7 +343,7 @@ class RemoteAutoPipelines(BaseAutoPipelines):
         for conn in training_data_reference:
             if self._workspace.wml_client.project_type == 'local_git_storage':
                 conn.location.userfs = 'true'
-            conn._wml_client = self._workspace.wml_client
+            conn.set_client(self._workspace.wml_client)
             # TODO: remove S3 implementation
             if conn.type == DataConnectionTypes.S3:
                 conn._validate_cos_resource()
@@ -357,7 +357,7 @@ class RemoteAutoPipelines(BaseAutoPipelines):
                 if isinstance(conn, DataConnection):
                     if self._workspace.wml_client.project_type == 'local_git_storage':
                         conn.location.userfs = 'true'
-                    conn._wml_client = self._workspace.wml_client
+                    conn.set_client(self._workspace.wml_client)
                     # TODO: remove S3 implementation
                     if conn.type == DataConnectionTypes.S3:
                         conn._validate_cos_resource()
@@ -404,7 +404,7 @@ class RemoteAutoPipelines(BaseAutoPipelines):
         # note: for FSLocation we are creating asset and changing location to AssetLocation
         # so href is not set properly, setter on wml_client will resolve that issue
         for conn in training_data_reference:
-            conn._wml_client = self._workspace.wml_client
+            conn.set_client(self._workspace.wml_client)
         # --- end note
 
         # note: if user did not provide results storage information, use default ones
@@ -434,6 +434,9 @@ class RemoteAutoPipelines(BaseAutoPipelines):
             elif isinstance(training_data_reference[0].location, ContainerLocation) and not self._workspace.wml_client.ICP:
                 training_results_reference = DataConnection(location=ContainerLocation(path="default_autoai_out"))
 
+            elif isinstance(training_data_reference[0].location, DatabaseLocation) and not self._workspace.wml_client.ICP:
+                training_results_reference = DataConnection(location=ContainerLocation(path="default_autoai_out"))
+
             else:
                 location = FSLocation()
                 if self._workspace.WMLS:
@@ -738,8 +741,9 @@ class RemoteAutoPipelines(BaseAutoPipelines):
         user_holdout_exists = True
         try:
             with DisableWarningsLogger():
-                with redirect_stdout(open(os.devnull, "w")):
-                    self.get_test_data_connections()
+                with open(os.devnull, "w") as f:
+                    with redirect_stdout(f):
+                        self.get_test_data_connections()
 
         except TestDataNotPresent:
             user_holdout_exists = False
@@ -754,7 +758,7 @@ class RemoteAutoPipelines(BaseAutoPipelines):
 
         for data_connection in data_connections:  # note: populate DataConnections with optimizer params
             data_connection.auto_pipeline_params = deepcopy(optimizer_parameters)
-            data_connection._wml_client = self._engine._wml_client
+            data_connection.set_client(self._engine._wml_client)
             data_connection._run_id = self._engine._current_run_id
             data_connection._user_holdout_exists = user_holdout_exists
 
@@ -788,7 +792,7 @@ class RemoteAutoPipelines(BaseAutoPipelines):
 
         for data_connection in data_connections:  # note: populate DataConnections with optimizer params
             data_connection.auto_pipeline_params = deepcopy(optimizer_parameters)
-            data_connection._wml_client = self._engine._wml_client
+            data_connection.set_client(self._engine._wml_client)
             data_connection._run_id = self._engine._current_run_id
             data_connection._test_data = True
             data_connection._user_holdout_exists = True
@@ -848,7 +852,7 @@ class RemoteAutoPipelines(BaseAutoPipelines):
 
             results_connection = DataConnection._from_dict(_dict=details['entity']['results_reference'])
             results_connection.auto_pipeline_params = deepcopy(optimizer_parameters)
-            results_connection._wml_client = self._engine._wml_client
+            results_connection.set_client(self._engine._wml_client)
             results_connection._run_id = self._engine._current_run_id
             results_connection._obm = True  # indicator for OBM output data
             results_connection._obm_cos_path = path
