@@ -419,7 +419,7 @@ class AutoPipelinesRuns(BaseAutoPipelinesRuns):
 
         for data_connection in data_connections:  # note: populate DataConnections with optimizer params
             data_connection.auto_pipeline_params = deepcopy(optimizer_parameters)
-            data_connection._wml_client = self._wml_engine._wml_client
+            data_connection.set_client(self._wml_engine._wml_client)
             data_connection._run_id = run_id
 
         return data_connections
