@@ -186,13 +186,13 @@ class LocalAutoPipelinesRuns(BaseAutoPipelinesRuns):
                     (self.training_result_reference.type == 'container' or
                     self.training_result_reference.type == 'data_asset' or
                      self.training_result_reference.type == 'connection_asset') and wml_client is not None):
-                    self.training_result_reference._wml_client = wml_client
+                    self.training_result_reference.set_client(wml_client)
 
                 for data_ref in self.training_data_reference:
                     if hasattr(data_ref, 'type') and (data_ref.type == 'container' or
                                                       data_ref.type == 'connection_asset' or
                                                       data_ref.type == 'data_asset') and wml_client is not None:
-                        data_ref._wml_client = wml_client
+                        data_ref.set_client(wml_client)
                 # --- end note
 
                 data_clients, result_client = prepare_cos_client(
