@@ -27,10 +27,13 @@ from warnings import warn
 import ibm_watson_machine_learning._wrappers.requests as requests
 from pandas import concat, DataFrame, read_csv
 
-from ibm_watson_machine_learning.utils.autoai.utils import try_load_dataset, try_load_tar_gz
+from ibm_watson_machine_learning.utils.autoai.utils import try_load_dataset, try_load_tar_gz, try_import_autoai_libs
 from ibm_watson_machine_learning.utils.autoai.enums import DataConnectionTypes
 from ibm_watson_machine_learning.utils.autoai.errors import CannotReadSavedRemoteDataBeforeFit, WrongAssetType
 from ibm_watson_machine_learning.wml_client_error import ApiRequestFailure, WMLClientError
+from ibm_watson_machine_learning.utils.utils import is_lib_installed
+from ibm_watson_machine_learning.data_loaders.experiment import ExperimentDataLoader
+from ibm_watson_machine_learning.data_loaders.datasets.experiment import ExperimentIterableDataset
 
 if TYPE_CHECKING:
     from ibm_boto3 import resource
@@ -52,6 +55,7 @@ class BaseDataConnection(ABC):
         self._obm = None
         self._obm_cos_path = None
         self.id = None
+        self._datasource_type = None
 
     @abstractmethod
     def _to_dict(self) -> dict:
@@ -188,10 +192,43 @@ class BaseDataConnection(ABC):
 
         return json_content
 
+    def _get_attachemnt_details(self, data_asset_id: str):
+        response = requests.get(self._wml_client.service_instance._href_definitions.get_data_asset_href(data_asset_id),
+                                params=self._wml_client._params(),
+                                headers=self._wml_client._get_headers())
+        data_asset_details = self._wml_client.data_assets._handle_response(200, u'Cannot get data asset details', response)
+
+        attachments_data_asset_details = data_asset_details.get('attachments', [{}])[0]
+        # note: Return attachment details from data asset details if it is not a connected data asset:
+        if attachments_data_asset_details and attachments_data_asset_details.get('connection_id') is None:
+            return attachments_data_asset_details
+        else:
+            attachment_id = attachments_data_asset_details.get('id')
+            if not self._wml_client.ICP and not self._wml_client.WSD:
+                response = requests.get(
+                    self._wml_client.service_instance._href_definitions.get_attachment_href(data_asset_id, attachment_id),
+                    params=self._wml_client.data_assets._client._params(),
+                    headers=self._wml_client.data_assets._client._get_headers())
+            else:
+                response = requests.get(
+                    self._wml_client.service_instance._href_definitions.get_attachment_href(data_asset_id, attachment_id),
+                    params=self._wml_client.data_assets._client._params(),
+                    headers=self._wml_client.data_assets._client._get_headers(),
+                    verify=False)
+
+            self._wml_client.data_assets._handle_response(200, u'Cannot get attachment details', response)
+
+            return response.json()
+
     def _check_if_connection_asset_is_s3(self) -> bool:
         try:
+            # fast return true if the connection is an instance of S3Connection and non s3 type
+            from .connections import S3Connection
+            if isinstance(self.connection, S3Connection) and self.type != 's3':
+                return True
+
             if self.type == 'data_asset':
-                if self.connection is not None:
+                if self.connection is not None and hasattr(self.connection, 'href'):
                     items = self.connection.href.split('/')
 
                 else:
@@ -199,30 +236,20 @@ class BaseDataConnection(ABC):
                 _id = items[-1].split('?')[0]
 
                 if self._wml_client is not None:
-                    data_asset_details = self._wml_client.data_assets.get_details(_id)
-
-                    attachment_id = data_asset_details['metadata']['attachment_id']
-                    response = requests.get(
-                        self._wml_client.service_instance._href_definitions.get_attachment_href(_id, attachment_id),
-                        params=self._wml_client.data_assets._client._params(),
-                        headers=self._wml_client.data_assets._client._get_headers())
-
-                    if response.status_code == 200:
-                        attachment_details = response.json()
-                        if 'connection_id' not in attachment_details:
-                            if 'handle' in attachment_details and 'bucket' in attachment_details['handle']:
-                                connection_details = self._create_conn_details_for_container()
-                                attachment_details['connection_path'] = attachment_details['handle']['key']
-
-                            else:
-                                return False
-
+                    attachment_details = self._get_attachemnt_details(_id)
+                    if 'connection_id' not in attachment_details:
+                        if 'handle' in attachment_details and 'bucket' in attachment_details['handle']:
+                            connection_details = self._create_conn_details_for_container()
+                            attachment_details['connection_path'] = '/' + attachment_details['handle']['bucket'] + '/' + attachment_details['handle']['key']
+                        elif 'url' in attachment_details and 'name' in attachment_details and 'bucket' in attachment_details:
+                            connection_details = self._create_conn_details_for_container()
+                            attachment_details['connection_path'] = '/' + attachment_details['bucket']['bucket_name'] + '/' + attachment_details['name']
                         else:
-                            connection_details = self._wml_client.connections.get_details(
-                                attachment_details['connection_id'])
+                            return False
 
                     else:
-                        raise WMLClientError("Failed while downloading the attachment " + attachment_id)
+                        connection_details = self._wml_client.connections.get_details(
+                            attachment_details['connection_id'])
 
                 else:
                     try:
@@ -272,6 +299,7 @@ class BaseDataConnection(ABC):
             # Note: Check with project libs if connection points to S3
             if self._wml_client is not None:
                 datasource_type = connection_details['entity']['datasource_type']
+                self._datasource_type = datasource_type
                 datasource_type_id_ibm_cos = self._wml_client.connections.get_datasource_type_uid_by_name(
                     'bluemixcloudobjectstorage')
                 datasource_type_id_aws_cos = self._wml_client.connections.get_datasource_type_uid_by_name(
@@ -310,6 +338,7 @@ class BaseDataConnection(ABC):
                             api_key=connection_details['entity']['properties'].get('api_key'),
                             auth_endpoint=connection_details['entity']['properties'].get('iam_url'),
                             endpoint_url=connection_details['entity']['properties'].get('url'),
+                            resource_instance_id=connection_details['entity']['properties'].get('resource_instance_id'),
                             _internal_use=True
                         )
 
@@ -327,6 +356,7 @@ class BaseDataConnection(ABC):
                         self.connection.api_key = connection_details['entity']['properties'].get('api_key')
                         self.connection.auth_endpoint = connection_details['entity']['properties'].get('iam_url')
                         self.connection.endpoint_url = connection_details['entity']['properties'].get('url')
+                        self.connection.resource_instance_id = connection_details['entity']['properties'].get('resource_instance_id')
 
                     else:
                         self.connection.secret_access_key = connection_details['entity']['properties'].get('secret_key')
@@ -338,8 +368,17 @@ class BaseDataConnection(ABC):
 
                 if self.type == 'data_asset':
                     try:
+                        # Workaround for connected data assets promoted from catalog
+                        if not attachment_details['connection_path'].startswith('/'):
+                            attachment_details['connection_path'] = '/' + attachment_details['connection_path']
                         self.location.bucket = attachment_details['connection_path'].split('/')[1]
-                        self.location.path = attachment_details['connection_path'].split('/')[2]
+                        self.location.id = None
+                        # Remove excel sheet name from connection_path if data asset is excel file:
+                        if self.auto_pipeline_params.get('excel_sheet') and \
+                                attachment_details['connection_path'].endswith("/"+str(self.auto_pipeline_params.get('excel_sheet'))):
+                            self.location.path = '/'.join(attachment_details['connection_path'].split('/')[2:-1])
+                        else:
+                            self.location.path = '/'.join(attachment_details['connection_path'].split('/')[2:])
 
                     except IndexError:
                         self.location.bucket = connection_details['entity']['properties']['bucket']
@@ -385,25 +424,6 @@ class BaseDataConnection(ABC):
 
         return data
 
-    def _download_data_from_cos(self, cos_client: 'resource') -> 'DataFrame':
-        """Download training data for this connection. COS version"""
-        location = self.location.file_name if hasattr(self.location, "file_name") else self.location.path
-        try:
-            file = cos_client.Object(self.location.bucket,
-                                     location).get()
-        except:
-            file = list(cos_client.Bucket(self.location.bucket).objects.filter(
-                Prefix=location))[0].get()
-
-        buffer = io.BytesIO(file['Body'].read())
-        data = try_load_dataset(buffer=buffer,
-                                sheet_name=self.auto_pipeline_params.get('excel_sheet', 0),
-                                separator=self.auto_pipeline_params.get('csv_separator', ','),
-                                encoding=self.auto_pipeline_params.get('encoding', 'utf-8')
-                                )
-
-        return data
-
     def _download_training_data_from_data_asset_storage(self) -> 'DataFrame':
         """Download training data for this connection. Data Storage."""
 
@@ -518,7 +538,7 @@ class BaseDataConnection(ABC):
 
             return data
 
-    def _download_training_data_from_file_system(self) -> 'DataFrame':
+    def _download_training_data_from_file_system(self, binary: bool = False) -> Tuple['DataFrame', bytes]:
         """Download training data for this connection. File system version."""
 
         try:
@@ -533,6 +553,10 @@ class BaseDataConnection(ABC):
                 raise ApiRequestFailure(u'Failure during {}.'.format("downloading model"), csv_response)
 
             downloaded_asset = csv_response.content
+
+            if binary:
+                return downloaded_asset
+
             # note: read the csv/xlsx file from the memory directly into the pandas DataFrame
             buffer = io.BytesIO(downloaded_asset)
             data = try_load_dataset(buffer=buffer,
@@ -542,6 +566,9 @@ class BaseDataConnection(ABC):
                                     )
         except (ApiRequestFailure, AttributeError):
             with open(self.location.path, 'rb') as data_buffer:
+                if binary:
+                    return data_buffer.read()
+
                 data = try_load_dataset(buffer=data_buffer,
                                         sheet_name=self.auto_pipeline_params.get('excel_sheet', 0),
                                         separator=self.auto_pipeline_params.get('csv_separator', ','),
@@ -671,6 +698,25 @@ class BaseDataConnection(ABC):
 
         return data
 
+    def _download_data_from_cos(self, cos_client: 'resource') -> 'DataFrame':
+        """Download training data for this connection. COS version"""
+        location = self.location.file_name if hasattr(self.location, "file_name") else self.location.path
+        try:
+            file = cos_client.Object(self.location.bucket,
+                                     location).get()
+        except:
+            file = list(cos_client.Bucket(self.location.bucket).objects.filter(
+                Prefix=location))[0].get()
+
+        buffer = io.BytesIO(file['Body'].read())
+        data = try_load_dataset(buffer=buffer,
+                                sheet_name=self.auto_pipeline_params.get('excel_sheet', 0),
+                                separator=self.auto_pipeline_params.get('csv_separator', ','),
+                                encoding=self.auto_pipeline_params.get('encoding', 'utf-8')
+                                )
+
+        return data
+
     def _create_conn_details_for_container(self) -> dict:
         if self._wml_client is not None:
             if self._wml_client.default_space_id is not None:
@@ -682,13 +728,15 @@ class BaseDataConnection(ABC):
 
                 response = requests.get(
                     project_url,
-                    headers=self._wml_client._get_headers()
+                    headers=self._wml_client._get_headers(projects_token=True)
                 )
 
                 details = response.json()
 
             properties = details['entity']['storage']['properties']
-            properties.update(details['entity']['storage']['properties']['credentials']['editor'])
+            creds = details['entity']['storage']['properties']['credentials'].get('admin')
+            creds = creds if creds else details['entity']['storage']['properties']['credentials'].get('editor')
+            properties.update(creds)
             properties['url'] = properties['endpoint_url']
             properties['access_key'] = properties.get('access_key_id')
             properties['secret_key'] = properties.get('secret_access_key')
@@ -744,3 +792,108 @@ class BaseDataConnection(ABC):
                 raise NotImplementedError(f"This functionality can be run only on Watson Studio.")
 
         return connection_details
+
+    def _download_data_from_flight_service(self,
+                                           data_location: 'DataConnection',
+                                           binary: bool = False,
+                                           flight_parameters: dict = None,
+                                           headers: dict = None):
+        is_lib_installed(lib_name='pyarrow', minimum_version='3.0.0', install=True)
+
+        if flight_parameters is None:
+            flight_parameters = {"num_partitions": 4}
+
+        dict_connection = data_location._to_dict()
+
+        experiment_metadata = {
+            "n_parallel_data_connections": flight_parameters.get("num_partitions", 4),
+            "prediction_column": self.auto_pipeline_params.get('prediction_column'),
+            "prediction_type": self.auto_pipeline_params.get('prediction_type'),
+            "project_id": self._wml_client.default_project_id,
+            "space_id": self._wml_client.default_space_id,
+            'headers': self._wml_client._get_headers() if headers is None else headers
+        }
+
+        experiment_metadata.update(self.auto_pipeline_params)
+
+        iterable_dataset = ExperimentIterableDataset(
+            connection=dict_connection,
+            with_subsampling=False,
+            experiment_metadata=experiment_metadata,
+            normal_read=True,
+            binary_data=binary,
+            flight_parameters=flight_parameters if flight_parameters is not None else {},
+            fallback_to_one_connection=False,
+            _wml_client=self._wml_client
+        )
+        data_loader = ExperimentDataLoader(dataset=iterable_dataset)
+
+        for data in data_loader:
+            return data
+
+    def _upload_data_via_flight_service(self,
+                                        data_location: 'DataConnection',
+                                        data: DataFrame = None,
+                                        file_path: str = None,
+                                        remote_name: str = None,
+                                        flight_parameters: dict = None,
+                                        headers: dict = None):
+        is_lib_installed(lib_name='pyarrow', minimum_version='3.0.0', install=True)
+
+        if flight_parameters is None:
+            flight_parameters = {"num_partitions": 1}
+
+        dict_connection = data_location._to_dict()
+
+        if remote_name:
+            if dict_connection.get('location', {}).get('path'):
+                dict_connection['location']['path'] = dict_connection['location']['path'] + '/' + remote_name
+                dict_connection['location']['path'] = dict_connection['location']['path'].replace('//', '/')
+            elif dict_connection.get('location', {}).get('file_name'):
+                actual_path = dict_connection['location']['file_name']
+                last_slash_index = actual_path.rfind('/') if '/' in actual_path else 0
+
+                if '.' in actual_path[last_slash_index:]:
+                    actual_path = actual_path[:-last_slash_index]
+
+                if actual_path:
+                    dict_connection['location']['path'] = actual_path + '/' + remote_name
+                    dict_connection['location']['path'] = dict_connection['location']['path'].replace('//', '/')
+                else:
+                    dict_connection['location']['path'] = remote_name
+            else:
+                dict_connection['location']['path'] = remote_name
+        elif dict_connection.get('location', {}).get('file_name'):
+            dict_connection['location']['path'] = dict_connection['location']['file_name']
+
+        experiment_metadata = {
+            "n_parallel_data_connections": flight_parameters.get("num_partitions", 1),
+            "project_id": self._wml_client.default_project_id,
+            "space_id": self._wml_client.default_space_id,
+            'headers': self._wml_client._get_headers() if headers is None else headers
+        }
+
+        experiment_metadata.update(self.auto_pipeline_params)
+
+        iterable_dataset = ExperimentIterableDataset(
+            connection=dict_connection,
+            experiment_metadata=experiment_metadata,
+            binary_data=True if file_path is not None else False,
+            flight_parameters=flight_parameters if flight_parameters is not None else {},
+            _wml_client=self._wml_client
+        )
+
+        if data is not None:
+            try:
+                iterable_dataset.write(data=data)
+
+            except Exception as e:
+                if 'gRPC message exceeds maximum size' in str(e):
+                    raise ValueError(f"Exceeds maximum data size. Please provide data file path "
+                                     f"instead of the pandas DataFrame to upload data in binary mode. Error: {e}")
+
+                else:
+                    raise e
+
+        else:
+            iterable_dataset.write(file_path=file_path)
