@@ -42,6 +42,7 @@ from warnings import warn
 from ibm_boto3 import resource
 from ibm_botocore.client import ClientError
 from pandas import DataFrame
+import pandas as pd
 import ibm_watson_machine_learning._wrappers.requests as requests
 
 from ibm_watson_machine_learning.utils.autoai.enums import PredictionType, DataConnectionTypes
@@ -59,10 +60,6 @@ from ibm_watson_machine_learning.wml_client_error import MissingValue, ApiReques
 from .base_connection import BaseConnection
 from .base_data_connection import BaseDataConnection
 from .base_location import BaseLocation
-from ..flight.service import FlightService
-from ..flight.utils import get_data_asset_attachment, try_import_pyarrow
-from ..flight.errors import DataSourceTypeNotRecognized
-from typing import Callable, Any
 
 if TYPE_CHECKING:
     from ibm_watson_machine_learning.workspace import WorkSpace
@@ -122,7 +119,7 @@ class DataConnection(BaseDataConnection):
             if kwargs.get('training_status') is not None:
                 location._training_status = kwargs['training_status']
 
-        elif connection_asset_id is not None and isinstance(location, (S3Location, DatabaseLocation)):
+        elif connection_asset_id is not None and isinstance(location, (S3Location, DatabaseLocation, NFSLocation)):
             connection = ConnectionAsset(connection_id=connection_asset_id)
 
         super().__init__()
@@ -134,17 +131,14 @@ class DataConnection(BaseDataConnection):
         if isinstance(connection, S3Connection):
             self.type = DataConnectionTypes.S3
 
-        elif isinstance(location, NFSLocation):
-            self.type = DataConnectionTypes.CA
-            if self.connection is not None:
-                self.location.id = self.connection.asset_id
-
         elif isinstance(connection, ConnectionAsset):
             self.type = DataConnectionTypes.CA
-            # note: We expect a `file_name` keyword for CA pointing to COS.
-            if isinstance(self.location, S3Location):
+            # note: We expect a `file_name` keyword for CA pointing to COS or NFS.
+            if isinstance(self.location, (S3Location, NFSLocation)):
                 self.location.file_name = self.location.path
                 del self.location.path
+                if isinstance(self.location, NFSLocation):
+                    del self.location.id
             # --- end note
 
         elif isinstance(location, FSLocation):
@@ -191,6 +185,21 @@ class DataConnection(BaseDataConnection):
         if isinstance(self.location, (AssetLocation, WSDAssetLocation)):
             self.location.wml_client = self.__wml_client
 
+    def set_client(self, wml_client):
+        """
+        Set initialized wml client in connection to enable write/read operations with connection to service.
+
+        Parameters
+        ----------
+        wml_client: APIClient, required
+            Wml client to connect to service.
+
+        Example
+        -------
+        >>> DataConnection.set_client(wml_client)
+        """
+        self._wml_client = wml_client
+
     # --- end note
 
     @classmethod
@@ -493,7 +502,8 @@ class DataConnection(BaseDataConnection):
              excel_sheet: Union[str, int] = 0,
              encoding: Optional[str] = 'utf-8',
              raw: Optional[bool] = False,
-             **kwargs) -> Union['DataFrame', Tuple['DataFrame', 'DataFrame']]:
+             binary: Optional[bool] = False,
+             **kwargs) -> Union['DataFrame', Tuple['DataFrame', 'DataFrame'], bytes]:
         """
         Download dataset stored in remote data storage.
 
@@ -512,18 +522,29 @@ class DataConnection(BaseDataConnection):
             Encoding type of the CSV
 
         raw: bool, optional
-            Default False. If False there will be applied simple data preprocessing (the same as in the backend).
+            Default False. If False there wil be applied simple data preprocessing (the same as in the backend).
             If True, data will be not preprocessed.
 
+        binary: bool, optional
+            Indicates to retrieve data in binary mode. The result will be a python binary type variable.
+            Default False.
+
         Returns
         -------
         pandas.DataFrame contains dataset from remote data storage : Xy_train
+
         or
+
         Tuple[pandas.DataFrame, pandas.DataFrame, pandas.DataFrame, pandas.DataFrame] : X_train, X_holdout, y_train, y_holdout
+
         or
+
         Tuple[pandas.DataFrame, pandas.DataFrame] : X_test, y_test
         containing training data and holdout data from remote storage.
 
+        or
+
+        bytes object
 
         Example
         -------
@@ -547,6 +568,17 @@ class DataConnection(BaseDataConnection):
         >>> train_data_connections = optimizer.get_data_connections()
         >>> data = train_connections[0].read() # only train data
         """
+
+        # enables flight automatically for CP4D 4.0.x, and cloud, for 3.0 and 3.5 we do not have a flight service there
+        use_flight = kwargs.get(
+            'use_flight',
+            bool((self._wml_client is not None or 'USER_ACCESS_TOKEN' in os.environ) and (
+                    self._wml_client.ICP_40 or self._wml_client.CLOUD_PLATFORM_SPACES)))
+
+        flight_parameters = kwargs.get('flight_parameters', {})
+
+        impersonate_header = kwargs.get('impersonate_header', None)
+
         if with_holdout_split and self._user_holdout_exists:  # when this connection is training one
             raise NoAutomatedHoldoutSplit(reason="Experiment was run based on user defined holdout dataset.")
 
@@ -587,7 +619,10 @@ class DataConnection(BaseDataConnection):
                     from project_lib import Project
 
                 except ModuleNotFoundError:
-                    raise NotImplementedError(f"This functionality can be run only on Watson Studio.")
+                    raise ConnectionError(
+                        "This functionality can be run only on Watson Studio or with wml_client passed to connection. "
+                        "Please initialize WML client using `DataConnection.set_client(wml_client)` function "
+                        "to be able to use this functionality.")
 
         if (with_holdout_split or self._test_data) and not self.auto_pipeline_params.get('prediction_type', False):
             raise MissingAutoPipelinesParameters(
@@ -606,66 +641,76 @@ class DataConnection(BaseDataConnection):
 
         data = DataFrame()
 
-        # TODO: Remove S3 implementation
+        headers = None
+        if self._wml_client is None:
+            token = os.environ.get('USER_ACCESS_TOKEN')
+            if token is not None:
+                headers = {'Authorization': f'Bearer {token}'}
+        elif impersonate_header is not None:
+            headers = self._wml_client._get_headers()
+            headers['impersonate'] = impersonate_header
+
         if self.type == DataConnectionTypes.S3:
-            warn(message="S3 DataConnection is deprecated! Please use data_asset_id instead.")
-
-            cos_client = self._init_cos_client()
-
-            try:
-                if self._obm:
-                    data = self._download_obm_data_from_cos(cos_client=cos_client)
-
-                else:
-                    data = self._download_data_from_cos(cos_client=cos_client)
-
-            except Exception as cos_access_exception:
-                raise ConnectionError(
-                    f"Unable to access data object in cloud object storage with credentials supplied. "
-                    f"Error: {cos_access_exception}")
+            raise ConnectionError(
+                f"S3 DataConnection is deprecated! Please use data_asset_id instead.")
 
         elif self.type == DataConnectionTypes.DS:
-            try:
-                with all_logging_disabled():
-                    if self._check_if_connection_asset_is_s3():
-                        cos_client = self._init_cos_client()
-
-                        if self._obm:
-                            data = self._download_obm_data_from_cos(cos_client=cos_client)
-
-                        else:
-                            data = self._download_data_from_cos(cos_client=cos_client)
-                    else:
+            if use_flight and not self._obm:
+                if self._check_if_connection_asset_is_s3():
+                    # note: update flight parameters only if `connection_properties` was not set earlier
+                    #       (e.x. by wml/autoi)
+                    if not flight_parameters.get('connection_properties'):
+                        flight_parameters = self._update_flight_parameters_with_connection_details(flight_parameters)
+                try:
+                    data = self._download_data_from_flight_service(data_location=self,
+                                                                   binary=binary,
+                                                                   flight_parameters=flight_parameters,
+                                                                   headers=headers)
+                except ConnectionError as flight_connection_error:
+                    # note: try to download normal data asset directly from cams - to keep backward compatibility
+                    if self._wml_client and not self._check_if_connection_asset_is_s3():
+                        import warnings
+                        warnings.warn(str(flight_connection_error), Warning)
                         data = self._download_training_data_from_data_asset_storage()
+                    else:
+                        raise flight_connection_error
 
-            except NotImplementedError as e:
-                raise e
 
-            except FileNotFoundError as e:
-                raise e
 
-            except Exception as e:
-                # do not try Flight if we are on the cloud
-                if self._wml_client is not None:
-                    if not self._wml_client.ICP:
-                        raise e
+            # backward compatibility
+            else:
+                try:
+                    with all_logging_disabled():
+                        if self._check_if_connection_asset_is_s3():
+                            cos_client = self._init_cos_client()
 
-                elif os.environ.get('USER_ACCESS_TOKEN') is None:
+                            if self._obm:
+                                data = self._download_obm_data_from_cos(cos_client=cos_client)
+
+                            else:
+                                data = self._download_data_from_cos(cos_client=cos_client)
+                        else:
+                            data = self._download_training_data_from_data_asset_storage()
+
+                except NotImplementedError as e:
                     raise e
 
-                data_location = self.location.to_dict()
-                data_location.update({'type': self.type})
+                except FileNotFoundError as e:
+                    raise e
 
-                try_import_pyarrow()
+                except Exception as e:
+                    # do not try Flight if we are on the cloud
+                    if self._wml_client is not None:
+                        if not self._wml_client.ICP:
+                            raise e
 
-                flight_service = FlightService(
-                    wml_client=self._wml_client,
-                    params=self.auto_pipeline_params,
-                    data_location=data_location,
-                    experiment_metadata=kwargs.get('experiment_metadata')
-                )
+                    elif os.environ.get('USER_ACCESS_TOKEN') is None:
+                        raise CannotReadSavedRemoteDataBeforeFit()
 
-                data = flight_service.read()
+                    data = self._download_data_from_flight_service(data_location=self,
+                                                                   binary=binary,
+                                                                   flight_parameters=flight_parameters,
+                                                                   headers=headers)
 
         elif self.type == DataConnectionTypes.FS:
 
@@ -675,51 +720,58 @@ class DataConnection(BaseDataConnection):
                 data = self._download_training_data_from_file_system()
 
         elif self.type == DataConnectionTypes.CA or self.type == DataConnectionTypes.CN:
-            try:
-                with all_logging_disabled():
-                    if self._check_if_connection_asset_is_s3():
-                        cos_client = self._init_cos_client()
+            if use_flight and not self._obm:
+                # Workaround for container connection type, we need to fetch COS details from space/project
+                if self.type == DataConnectionTypes.CN:
+                    # note: update flight parameters only if `connection_properties` was not set earlier
+                    #       (e.x. by wml/autoi)
+                    if not flight_parameters.get('connection_properties'):
+                        flight_parameters = self._update_flight_parameters_with_connection_details(flight_parameters)
 
-                        try:
-                            if self._obm:
-                                data = self._download_obm_data_from_cos(cos_client=cos_client)
+                data = self._download_data_from_flight_service(data_location=self,
+                                                               binary=binary,
+                                                               flight_parameters=flight_parameters,
+                                                               headers=headers)
 
-                            else:
-                                data = self._download_data_from_cos(cos_client=cos_client)
+            else:  # backward compatibility
+                try:
+                    with all_logging_disabled():
+                        if self._check_if_connection_asset_is_s3():
+                            cos_client = self._init_cos_client()
+                            try:
+                                if self._obm:
+                                    data = self._download_obm_data_from_cos(cos_client=cos_client)
 
-                        except Exception as cos_access_exception:
-                            raise ConnectionError(
-                                f"Unable to access data object in cloud object storage with credentials supplied. "
-                                f"Error: {cos_access_exception}")
+                                else:
+                                    data = self._download_data_from_cos(cos_client=cos_client)
 
-                    else:
-                        data = self._download_data_from_nfs_connection()
+                            except Exception as cos_access_exception:
+                                raise ConnectionError(
+                                    f"Unable to access data object in cloud object storage with credentials supplied. "
+                                    f"Error: {cos_access_exception}")
+                        else:
+                            data = self._download_data_from_nfs_connection()
 
-            except Exception as e:
-                # do not try Flight is we are on the cloud
-                if self._wml_client is not None:
-                    if not self._wml_client.ICP:
-                        raise e
+                except Exception as e:
+                    # do not try Flight is we are on the cloud
+                    if self._wml_client is not None:
+                        if not self._wml_client.ICP:
+                            raise e
 
-                elif os.environ.get('USER_ACCESS_TOKEN') is None:
-                    raise e
+                    elif os.environ.get('USER_ACCESS_TOKEN') is None:
+                        raise CannotReadSavedRemoteDataBeforeFit()
 
-                try_import_pyarrow()
-
-                flight_service = FlightService(
-                    wml_client=self._wml_client,
-                    params=self.auto_pipeline_params,
-                    data_location={'location': self.location.to_dict(),
-                                   'type': self.type,
-                                   "connection": self.connection.to_dict()},
-                    experiment_metadata=kwargs.get('experiment_metadata')
-                )
-
-                data = flight_service.read()
+                    data = self._download_data_from_flight_service(data_location=self,
+                                                                   binary=binary,
+                                                                   flight_parameters=flight_parameters,
+                                                                   headers=headers)
 
         if isinstance(data, DataFrame) and 'Unnamed: 0' in data.columns.tolist():
             data.drop(['Unnamed: 0'], axis=1, inplace=True)
 
+        if binary:
+            return data
+
         if raw or (self.auto_pipeline_params.get('prediction_column') is None
                    and self.auto_pipeline_params.get('prediction_columns') is None):
             return data
@@ -750,77 +802,134 @@ class DataConnection(BaseDataConnection):
         remote_name: str, required
             Name that dataset should be stored with in remote data storage.
         """
-        # TODO: Remove S3 implementation
-        if self.type == DataConnectionTypes.S3:
-            warn(message="S3 DataConnection is deprecated! Please use data_asset_id instead.")
-            if remote_name is None:
-                raise MissingValue('remote_name')
+        # enables flight automatically for CP4D 4.0.x and cloud, for 3.0 and 3.5 we do not have a flight service there
+        use_flight = kwargs.get(
+            'use_flight',
+            bool((self._wml_client is not None or 'USER_ACCESS_TOKEN' in os.environ) and (
+                    self._wml_client.ICP_40 or self._wml_client.CLOUD_PLATFORM_SPACES)))
 
-            cos_resource_client = self._init_cos_client()
-            if isinstance(data, str):
-                with open(data, "rb") as file_data:
-                    cos_resource_client.Object(self.location.bucket, remote_name).upload_fileobj(
-                        Fileobj=file_data)
+        flight_parameters = kwargs.get('flight_parameters', {})
 
-            elif isinstance(data, DataFrame):
-                # note: we are saving csv in memory as a file and stream it to the COS
-                buffer = io.StringIO()
-                data.to_csv(buffer, index=False)
-                buffer.seek(0)
+        impersonate_header = kwargs.get('impersonate_header', None)
 
-                with buffer as f:
-                    cos_resource_client.Object(self.location.bucket, remote_name).upload_fileobj(
-                        Fileobj=io.BytesIO(bytes(f.read().encode())))
+        headers = None
+        if self._wml_client is None:
+            token = os.environ.get('USER_ACCESS_TOKEN')
+            if token is None:
+                raise ConnectionError("WML client missing. Please initialize WML client and pass it to "
+                                      "DataConnection._wml_client property to be able to use this functionality.")
 
             else:
-                raise TypeError("data should be either of type \"str\" or \"pandas.DataFrame\"")
+                headers = {'Authorization': f'Bearer {token}'}
+        elif impersonate_header is not None:
+            headers = self._wml_client._get_headers()
+            headers['impersonate'] = impersonate_header
+
+        # TODO: Remove S3 implementation
+        if self.type == DataConnectionTypes.S3:
+            raise ConnectionError("S3 DataConnection is deprecated! Please use data_asset_id instead.")
 
         elif self.type == DataConnectionTypes.CA or self.type == DataConnectionTypes.CN:
             if self._check_if_connection_asset_is_s3():
-                cos_resource_client = self._init_cos_client()
+                # do not try Flight if we are on the cloud
+                if self._wml_client is not None and not self._wml_client.ICP and not use_flight:  # CLOUD
+                    cos_resource_client = self._init_cos_client()
+                    if isinstance(data, str):
+                        with open(data, "rb") as file_data:
+                            cos_resource_client.Object(self.location.bucket, remote_name).upload_fileobj(
+                                Fileobj=file_data)
+
+                    elif isinstance(data, DataFrame):
+                        # note: we are saving csv in memory as a file and stream it to the COS
+                        buffer = io.StringIO()
+                        data.to_csv(buffer, index=False)
+                        buffer.seek(0)
+
+                        with buffer as f:
+                            cos_resource_client.Object(self.location.bucket, remote_name).upload_fileobj(
+                                Fileobj=io.BytesIO(bytes(f.read().encode())))
+
+                    else:
+                        raise TypeError("data should be either of type \"str\" or \"pandas.DataFrame\"")
+                # CP4D
+                else:
+                    # Workaround for container connection type, we need to fetch COS details from space/project
+                    if self.type == DataConnectionTypes.CN:
+                        # note: update flight parameters only if `connection_properties` was not set earlier
+                        #       (e.x. by wml/autoi)
+                        if not flight_parameters.get('connection_properties'):
+                            flight_parameters = self._update_flight_parameters_with_connection_details(flight_parameters)
+
+                    if isinstance(data, str):
+                        self._upload_data_via_flight_service(file_path=data,
+                                                             data_location=self,
+                                                             remote_name=remote_name,
+                                                             flight_parameters=flight_parameters,
+                                                             headers=headers)
+
+                    elif isinstance(data, DataFrame):
+                        # note: we are saving csv in memory as a file and stream it to the COS
+                        self._upload_data_via_flight_service(data=data,
+                                                             data_location=self,
+                                                             remote_name=remote_name,
+                                                             flight_parameters=flight_parameters,
+                                                             headers=headers)
+
+                    else:
+                        raise TypeError("data should be either of type \"str\" or \"pandas.DataFrame\"")
+
+            else:
+                if self._wml_client is not None and not self._wml_client.ICP and not use_flight:  # CLOUD
+                    raise ConnectionError("Connections other than COS are not supported on a cloud yet.")
+                # CP4D
+                else:
+                    if isinstance(data, str):
+                        self._upload_data_via_flight_service(file_path=data,
+                                                             data_location=self,
+                                                             remote_name=remote_name,
+                                                             flight_parameters=flight_parameters,
+                                                             headers=headers)
+
+                    elif isinstance(data, DataFrame):
+                        # note: we are saving csv in memory as a file and stream it to the COS
+                        self._upload_data_via_flight_service(data=data,
+                                                             data_location=self,
+                                                             remote_name=remote_name,
+                                                             flight_parameters=flight_parameters,
+                                                             headers=headers)
+
+                    else:
+                        raise TypeError("data should be either of type \"str\" or \"pandas.DataFrame\"")
+
+        elif self.type == DataConnectionTypes.DS:
+            if self._wml_client is not None and not self._wml_client.ICP and not use_flight:  # CLOUD
+                raise ConnectionError("Write of data for Data Asset is not supported on Cloud.")
+
+            elif self._wml_client is not None:
                 if isinstance(data, str):
-                    with open(data, "rb") as file_data:
-                        cos_resource_client.Object(self.location.bucket, remote_name).upload_fileobj(
-                            Fileobj=file_data)
+                    self._upload_data_via_flight_service(file_path=data,
+                                                         data_location=self,
+                                                         remote_name=remote_name,
+                                                         flight_parameters=flight_parameters,
+                                                         headers=headers)
 
                 elif isinstance(data, DataFrame):
                     # note: we are saving csv in memory as a file and stream it to the COS
-                    buffer = io.StringIO()
-                    data.to_csv(buffer, index=False)
-                    buffer.seek(0)
-
-                    with buffer as f:
-                        cos_resource_client.Object(self.location.bucket, remote_name).upload_fileobj(
-                            Fileobj=io.BytesIO(bytes(f.read().encode())))
+                    self._upload_data_via_flight_service(data=data,
+                                                         data_location=self,
+                                                         remote_name=remote_name,
+                                                         flight_parameters=flight_parameters,
+                                                         headers=headers)
 
                 else:
                     raise TypeError("data should be either of type \"str\" or \"pandas.DataFrame\"")
 
             else:
-                try_import_pyarrow()
-
-                flight_service = FlightService(
-                    wml_client=self._wml_client,
-                    params=self.auto_pipeline_params,
-                    data_location={'location': self.location.to_dict(),
-                                   'type': self.type,
-                                   "connection": self.connection.to_dict()},
-                    experiment_metadata=kwargs.get('experiment_metadata')
-                )
-
-                if isinstance(data, str):
-                    raise TypeError(f"'data' should be of pandas.DataFrame type if "
-                                    f"you want to store it with connection asset.")
-
-                elif isinstance(data, DataFrame):
-                    flight_service.write_data(data)
-
-        elif self.type == DataConnectionTypes.DS:
-            raise UseWMLClient('DataConnection.write()',
-                               reason="If you want to upload any data to CP4D instance, "
-                                      "firstly please get the WML client by calling "
-                                      "\"client = WMLInstance().get_client()\" "
-                                      "then call the method: \"client.data_assets.create()\"")
+                self._upload_data_via_flight_service(data=data,
+                                                     data_location=self,
+                                                     remote_name=remote_name,
+                                                     flight_parameters=flight_parameters,
+                                                     headers=headers)
 
     def _init_cos_client(self) -> 'resource':
         """Initiate COS client for further usage."""
@@ -851,6 +960,27 @@ class DataConnection(BaseDataConnection):
         except Exception as e:
             raise NotExistingCOSResource(self.location.bucket, self.location.path)
 
+    def _update_flight_parameters_with_connection_details(self, flight_parameters):
+        with all_logging_disabled():
+            self._check_if_connection_asset_is_s3()
+            connection_properties = {
+                "bucket": self.location.bucket,
+                "url": self.connection.endpoint_url
+            }
+            if hasattr(self.connection, 'auth_endpoint') and hasattr(self.connection, 'api_key'):
+                connection_properties["iam_url"] = self.connection.auth_endpoint
+                connection_properties["api_key"] = self.connection.api_key
+                connection_properties["resource_instance_id"] = self.connection.resource_instance_id
+
+            else:
+                connection_properties["secret_key"] = self.connection.secret_access_key
+                connection_properties["access_key"] = self.connection.access_key_id
+
+            flight_parameters.update({"connection_properties": connection_properties})
+            flight_parameters.update({"datasource_type": {"entity": {"name": self._datasource_type}}})
+
+        return flight_parameters
+
 
 # TODO: Remove S3 Implementation for connection
 class S3Connection(BaseConnection):
@@ -879,7 +1009,8 @@ class S3Connection(BaseConnection):
     """
 
     def __init__(self, endpoint_url: str, access_key_id: str = None, secret_access_key: str = None,
-                 api_key: str = None, service_name: str = None, auth_endpoint: str = None, _internal_use=False) -> None:
+                 api_key: str = None, service_name: str = None, auth_endpoint: str = None,
+                 resource_instance_id: str = None, _internal_use=False) -> None:
         if not _internal_use:
             warn(message="S3 DataConnection is deprecated! Please use data_asset_id instead.")
 
@@ -905,6 +1036,9 @@ class S3Connection(BaseConnection):
         if endpoint_url is not None:
             self.endpoint_url = endpoint_url
 
+        if resource_instance_id is not None:
+            self.resource_instance_id = resource_instance_id
+
 
 class S3Location(BaseLocation):
     """
@@ -1315,6 +1449,7 @@ class NFSLocation(BaseLocation):
     def __init__(self, path: str):
         self.path = path
         self.id = None
+        self.file_name = None
 
     def _get_file_size(self, workspace: 'Workspace', *args) -> 'int':
         params = workspace.wml_client._params().copy()
@@ -1328,6 +1463,12 @@ class NFSLocation(BaseLocation):
             raise Exception(u"Failure during getting asset details", asset_info_response.json())
         return asset_info_response.json()['details']['file_size']
 
+    def get_location(self) -> str:
+        if hasattr(self, "file_name"):
+            return self.file_name
+        else:
+            return self.path
+
 
 class CP4DAssetLocation(AssetLocation):
     """
