@@ -182,12 +182,12 @@ def is_uid(s):
 
 
 class HrefDefinitions:
-    def __init__(self, wml_credentials, cloud_platform_spaces=False, platform_url=None, cams_url=None,
-                 cp4d_platform_spaces=False, git_based_project=False):
-        self._wml_credentials = wml_credentials
+    def __init__(self, client, cloud_platform_spaces=False, platform_url=None, cams_url=None,
+                 cp4d_platform_spaces=False):
+        self._wml_credentials = client.wml_credentials
+        self._client = client
         self.cloud_platform_spaces = cloud_platform_spaces
         self.cp4d_platform_spaces = cp4d_platform_spaces
-        self.git_based_project = git_based_project
         self.platform_url = platform_url
         self.cams_url = cams_url
 
@@ -196,6 +196,9 @@ class HrefDefinitions:
         else:
             self.prepend = ''
 
+    def _is_git_based_project(self):
+        return self._client.project_type == "local_git_storage"
+
     def _get_platform_url_if_exists(self):
         return self.platform_url if self.platform_url else self._wml_credentials['url']
 
@@ -351,75 +354,75 @@ class HrefDefinitions:
         return MEMBERS_HREF_PATTERN.format(self._wml_credentials['url'],spaces_uid)
 
     def get_data_asset_href(self,asset_id):
-        return (ASSET if not self.git_based_project else GIT_BASED_PROJECT_ASSET)\
+        return (ASSET if not self._is_git_based_project() else GIT_BASED_PROJECT_ASSET)\
             .format(self._get_cams_url_if_exists(), asset_id)
 
     def get_data_assets_href(self):
-        return (ASSETS if not self.git_based_project else GIT_BASED_PROJECT_ASSETS)\
+        return (ASSETS if not self._is_git_based_project() else GIT_BASED_PROJECT_ASSETS)\
             .format(self._get_cams_url_if_exists())
 
     def get_assets_href(self):
-        return (ASSETS if not self.git_based_project else GIT_BASED_PROJECT_ASSETS)\
+        return (ASSETS if not self._is_git_based_project() else GIT_BASED_PROJECT_ASSETS)\
             .format(self._get_cams_url_if_exists())
 
     def get_asset_href(self,asset_id):
-        return (ASSET if not self.git_based_project else GIT_BASED_PROJECT_ASSET)\
+        return (ASSET if not self._is_git_based_project() else GIT_BASED_PROJECT_ASSET)\
             .format(self._get_cams_url_if_exists(), asset_id)
 
     def get_base_asset_href(self,asset_id):
-        return (ASSET if not self.git_based_project else GIT_BASED_PROJECT_ASSET).format('', asset_id)
+        return (ASSET if not self._is_git_based_project() else GIT_BASED_PROJECT_ASSET).format('', asset_id)
 
     def get_base_assets_href(self):
-        return (ASSETS if not self.git_based_project else GIT_BASED_PROJECT_ASSETS).format('')
+        return (ASSETS if not self._is_git_based_project() else GIT_BASED_PROJECT_ASSETS).format('')
 
     def get_base_asset_with_type_href(self,asset_type, asset_id):
-        return (ASSET if not self.git_based_project else GIT_BASED_PROJECT_ASSET).format('', asset_type) + '/' + asset_id
+        return (ASSET if not self._is_git_based_project() else GIT_BASED_PROJECT_ASSET).format('', asset_type) + '/' + asset_id
 
     def get_attachment_href(self,asset_id,attachment_id):
-        return (ATTACHMENT if not self.git_based_project else GIT_BASED_PROJECT_ATTACHMENT)\
+        return (ATTACHMENT if not self._is_git_based_project() else GIT_BASED_PROJECT_ATTACHMENT)\
             .format(self._get_cams_url_if_exists(), asset_id, attachment_id)
 
     def get_attachments_href(self, asset_id):
-        return (ATTACHMENTS if not self.git_based_project else GIT_BASED_PROJECT_ATTACHMENTS)\
+        return (ATTACHMENTS if not self._is_git_based_project() else GIT_BASED_PROJECT_ATTACHMENTS)\
             .format(self._get_cams_url_if_exists(), asset_id)
 
     def get_attachment_complete_href(self,asset_id,attachment_id):
-        return (ATTACHMENT_COMPLETE if not self.git_based_project else GIT_BASED_PROJECT_ATTACHMENT_COMPLETE)\
+        return (ATTACHMENT_COMPLETE if not self._is_git_based_project() else GIT_BASED_PROJECT_ATTACHMENT_COMPLETE)\
             .format(self._get_cams_url_if_exists(), asset_id, attachment_id)
 
     def get_search_asset_href(self):
-        return (SEARCH_DATA_ASSETS if not self.git_based_project else GIT_BASED_PROJECT_SEARCH_DATA_ASSETS)\
+        return (SEARCH_DATA_ASSETS if not self._is_git_based_project() else GIT_BASED_PROJECT_SEARCH_DATA_ASSETS)\
             .format(self._get_cams_url_if_exists())
 
     def get_search_shiny_href(self):
-        return (SEARCH_SHINY if not self.git_based_project else GIT_BASED_PROJECT_SEARCH_SHINY)\
+        return (SEARCH_SHINY if not self._is_git_based_project() else GIT_BASED_PROJECT_SEARCH_SHINY)\
             .format(self._get_cams_url_if_exists())
 
     def get_search_script_href(self):
-        return (SEARCH_SCRIPT if not self.git_based_project else GIT_BASED_PROJECT_SEARCH_SCRIPT)\
+        return (SEARCH_SCRIPT if not self._is_git_based_project() else GIT_BASED_PROJECT_SEARCH_SCRIPT)\
             .format(self._get_cams_url_if_exists())
 
     def get_model_definition_assets_href(self):
-        return (ASSETS if not self.git_based_project else GIT_BASED_PROJECT_ASSETS)\
+        return (ASSETS if not self._is_git_based_project() else GIT_BASED_PROJECT_ASSETS)\
             .format(self._get_cams_url_if_exists())
 
     def get_model_definition_search_asset_href(self):
-        return (SEARCH_MODEL_DEFINITIONS if not self.git_based_project else GIT_BASED_PROJECT_SEARCH_MODEL_DEFINITIONS)\
+        return (SEARCH_MODEL_DEFINITIONS if not self._is_git_based_project() else GIT_BASED_PROJECT_SEARCH_MODEL_DEFINITIONS)\
             .format(self._get_cams_url_if_exists())
 
     def get_wsd_model_href(self):
-        return (ASSETS if not self.git_based_project else GIT_BASED_PROJECT_ASSETS).format(self._wml_credentials['url'])
+        return (ASSETS if not self._is_git_based_project() else GIT_BASED_PROJECT_ASSETS).format(self._wml_credentials['url'])
 
     def get_wsd_model_attachment_href(self):
-        return (ASSET_FILES if not self.git_based_project else GIT_BASED_PROJECT_ASSET_FILES)\
+        return (ASSET_FILES if not self._is_git_based_project() else GIT_BASED_PROJECT_ASSET_FILES)\
             .format(self._wml_credentials['url'])
 
     def get_wsd_asset_search_href(self, asset_type):
-        return (SEARCH_ASSETS if not self.git_based_project else GIT_BASED_PROJECT_SEARCH_ASSETS)\
+        return (SEARCH_ASSETS if not self._is_git_based_project() else GIT_BASED_PROJECT_SEARCH_ASSETS)\
             .format(self._wml_credentials['url'], asset_type)
 
     def get_wsd_asset_type_href(self):
-        return (ASSET_TYPE if not self.git_based_project else GIT_BASED_PROJECT_ASSET_TYPE)\
+        return (ASSET_TYPE if not self._is_git_based_project() else GIT_BASED_PROJECT_ASSET_TYPE)\
             .format(self._wml_credentials['url'])
 
     def get_wsd_base_href(self):
