@@ -1012,7 +1012,7 @@ class ModelCollection:
 
     def _wsd_create_model_asset(self, url, input_payload, artifact, params, headers):
         import json
-        import requests
+        import ibm_watson_machine_learning._wrappers.requests as requests
         import urllib
         params = params
         headers = headers
@@ -1030,8 +1030,7 @@ class ModelCollection:
                 self.client.service_instance._href_definitions.get_wsd_asset_type_href(),
                 params=params,
                 data=atype_payload,
-                headers=aheaders,
-                verify=False
+                headers=aheaders
             )
             if asset_type_response.status_code != 200 and \
                     asset_type_response.status_code != 201 and asset_type_response.status_code != 409:
@@ -1041,8 +1040,7 @@ class ModelCollection:
                 self.client.service_instance._href_definitions.get_assets_href(),
                 params=params,
                 json=input_payload,
-                headers=headers,
-                verify=False
+                headers=headers
             )
             if create_response.status_code == 201:
                 try:
@@ -1065,8 +1063,7 @@ class ModelCollection:
                 response = requests.put(
                     content_upload_url,
                     files={'file': ('native', fdata, 'application/gzip', {'Expires': '0'})},
-                    params=params,
-                    verify=False
+                    params=params
                 )
                 if response.status_code == 201:
                     # update the attachement url with details :
@@ -1081,8 +1078,7 @@ class ModelCollection:
                     attach_response = requests.post(attach_url,
                                                     data=attach_payload,
                                                     params=params,
-                                                    headers=headers,
-                                                    verify=False)
+                                                    headers=headers)
                     if attach_response.status_code == 201:
                         artifact.reader().close()
                         return asset_details
