@@ -42,7 +42,8 @@ def process_heartbeat(client, **kwargs):
 
         if service_version.startswith("4.0"):
             version_details = service_version.split('-')
-            #use cpd403 for cpd4.0.x version for now
+            if version_details[1] >= "202202":
+                return "cpd407"
             if version_details[1] >= "202112":
                 return "cpd406"
             elif version_details[1] >= "202110":
@@ -64,7 +65,7 @@ def process_heartbeat(client, **kwargs):
 
 def import_diff(module_file_path):
 
-    if "sklearn023" in module_file_path:
+    if "py38-tf24-pt17-sk23" in module_file_path:
         return
     pathlist = Path(module_file_path).rglob('*.py')
     for path in pathlist:
@@ -74,21 +75,23 @@ def import_diff(module_file_path):
             module_name = ("ibmfl" + path_in_str.split("ibmfl")[2].replace("/", "."))[:-3]
             module_spec = importlib.util.spec_from_file_location(
                 module_name, path_in_str)
+            loader = importlib.util.LazyLoader(module_spec.loader)
+            module_spec.loader = loader
             module = importlib.util.module_from_spec(module_spec)
             sys.modules[module_name] = module
             module_spec.loader.exec_module(module)
 
 
-def choose_sklearn_version(software_spec):
+def choose_software_version(software_spec):
     if software_spec == "runtime-22.1-py3.9" or software_spec == "12b83a17-24d8-5082-900f-0ab31fbfd3cb":
-        return "sklearn102"
+        return "py39-tf27-pt110-sk10"
     else:
-        return "sklearn023"
+        return "py38-tf24-pt17-sk23"
 
 
 def check_python_version(platform_env):
     py_version = platform.python_version()
-    if platform_env != "cloud":
+    if platform_env != "cloud" and platform_env != "cpd407":
         if not py_version.startswith("3.7.") and not py_version.startswith("3.8."):
             raise Exception("Python version {} is not supported for Federated Learning. "
                             "Parties must use a supported Python version to train the model. "
@@ -117,8 +120,9 @@ class Party(WMLResource):
         'cpd402': "1.0.165",
         'cpd403': "/cpd403/ibmfl",
         'cpd406': "/cpd406/ibmfl",
-        'sklearn023': "/sklearn023/ibmfl",
-        'sklearn102': "/sklearn102/ibmfl"
+        'cpd407': "/cpd406/ibmfl",
+        'py38-tf24-pt17-sk23': "/py38-tf24-pt17-sk23/ibmfl", #default
+        'py39-tf27-pt110-sk10': "/py39-tf27-pt110-sk10/ibmfl"
     }
 
     def __init__(self, client=None, **kwargs):
@@ -164,7 +168,7 @@ class Party(WMLResource):
         #install the general lib (which is the cloud version)
         module_name = 'ibmfl'
         module_spec = importlib.util.spec_from_file_location(
-            module_name, ibmfl_base_module_location + '/sklearn023/ibmfl/__init__.py')
+            module_name, ibmfl_base_module_location + '/py38-tf24-pt17-sk23/ibmfl/__init__.py')
         module = importlib.util.module_from_spec(module_spec)
         sys.modules[module_name] = module
         module_spec.loader.exec_module(module)
@@ -239,14 +243,14 @@ class Party(WMLResource):
                 raise FLException("The current state of training is %s, so the party is not able to start a job." % state)
 
             if ready:
-                if "cpd406" in self.module_location:
+                if "cpd406" in self.module_location or "cloud" in self.module_location:
                     details = self._client.training.get_details(aggregator_id)
                     fl_entity = details["entity"]["federated_learning"]
                     if "software_spec" in fl_entity:
                         software_spec = fl_entity["software_spec"]["name"] if "name" in fl_entity["software_spec"] else fl_entity["software_spec"]["id"]
                     else:
                         software_spec = "default_py3.8"
-                    platform_env = choose_sklearn_version(software_spec)
+                    platform_env = choose_software_version(software_spec)
                     print("Loading {} environment..".format(platform_env))
                     self.module_location = '/'.join(self.module_location.split('/')[0:-2]) + self.SUPPORTED_PLATFORMS_MAP.get(platform_env)
                 import_diff(self.module_location)
