@@ -493,12 +493,9 @@ class Repository(WMLResource):
 
         if type == "":
             raise WMLClientError('Unsupported artifact type. Supported artifact types are models, libraries, runtimes, experiments, pipelines and functions')
-        if not self._ICP:
-            response_post = requests.post(url, json=clone_meta,
-                                          headers=self._client._get_headers())
-        else:
-            response_post = requests.post(url, json=clone_meta,
-                                          headers=self._client._get_headers())
+
+        response_post = requests.post(url, json=clone_meta,
+                                      headers=self._client._get_headers())
 
         details = self._handle_response(expected_status_code=200, operationName=u'cloning '+ type,
                                         response=response_post)
@@ -945,12 +942,7 @@ class Repository(WMLResource):
             # This doesn't seem to be done for CP4D 3.0.1 and before. We should do this for CP4D 3.5
             params.update({'purge_on_delete': 'true'})
 
-        if not self._ICP:
-            response = requests.delete(self._client.service_instance._href_definitions.get_asset_href(artifact_uid),
-                                           params=params,
-                                           headers=self._client._get_headers())
-        else:
-            response = requests.delete(self._client.service_instance._href_definitions.get_asset_href(artifact_uid),
+        response = requests.delete(self._client.service_instance._href_definitions.get_asset_href(artifact_uid),
                                        params=params,
                                        headers=self._client._get_headers())
 
@@ -2354,4 +2346,26 @@ class Repository(WMLResource):
         """
         self._client.experiments.list_revisions(experiment_uid, limit=limit)
 
+    def promote_model(self, model_id: str, source_project_id: str, target_space_id: str):
+        """
+           Promote model from project to space. Supported only for IBM Cloud Pak® for Data.
+
+           :param model_id: stored model
+           :type model_id: {str_type}
+
+           :param source_project_id: stored model
+           :type source_project_id: {str_type}
+
+           :param target_space_id: stored model
+           :type target_space_id: {str_type}
+
+           :returns: promoted model id
+           :rtype: str
+
+           A way you might use me is:
+
+           >>> promoted_model_id = client.models.promote(model_id, source_project_id, target_space_id)
+        """
+        return self._client._models.promote(model_id, source_project_id, target_space_id)
+
 
