@@ -205,7 +205,7 @@ class Shiny(WMLResource):
         else:
 
             creation_response = ""
-            if not self._ICP:
+            if not self._ICP or self._client.ICP_PLATFORM_SPACES:
                 creation_response = requests.post(
                         self._client.service_instance._href_definitions.get_data_assets_href(),
                         headers=self._client._get_headers(),
@@ -213,28 +213,20 @@ class Shiny(WMLResource):
                         json=asset_meta
                 )
             else:
-                if self._client.ICP_PLATFORM_SPACES:
+                # Until global asset for shiny is created
+                asset_type_response = requests.post(
+                    self._client.service_instance._href_definitions.get_wsd_asset_type_href() + '?',
+                    headers=self._client._get_headers(),
+                    json=base_shiny_asset,
+                    params=self._client._params()
+                )
+                if asset_type_response.status_code == 201 or asset_type_response.status_code == 409:
                     creation_response = requests.post(
                         self._client.service_instance._href_definitions.get_data_assets_href(),
                         headers=self._client._get_headers(),
-                        params=self._client._params(),
-                        json=asset_meta
-                    )
-                else:
-                    # Until global asset for shiny is created
-                    asset_type_response = requests.post(
-                        self._client.service_instance._href_definitions.get_wsd_asset_type_href() + '?',
-                        headers=self._client._get_headers(),
-                        json=base_shiny_asset,
+                        json=asset_meta,
                         params=self._client._params()
                     )
-                    if asset_type_response.status_code == 201 or asset_type_response.status_code == 409:
-                        creation_response = requests.post(
-                            self._client.service_instance._href_definitions.get_data_assets_href(),
-                            headers=self._client._get_headers(),
-                            json=asset_meta,
-                            params=self._client._params()
-                        )
 
             asset_details = self._handle_response(201, u'creating new asset', creation_response)
             #Step2: Create attachment
@@ -271,20 +263,13 @@ class Shiny(WMLResource):
                         )
                     if put_response.status_code == 201 or put_response.status_code == 200:
                         # Step4: Complete attachment
-                        if not self._ICP:
-                            complete_response = requests.post(
-                                # self._client.service_instance._href_definitions.get_attachment_href(asset_id,attachment_id)+"/complete",
-                                self._client.service_instance._href_definitions.get_attachment_complete_href(asset_id, attachment_id),
-                                headers=self._client._get_headers(),
-                                params = self._client._params()
+                        complete_response = requests.post(
+                            self._client.service_instance._href_definitions.get_attachment_complete_href(asset_id, attachment_id),
+                            headers=self._client._get_headers(),
+                            params = self._client._params()
+
+                        )
 
-                            )
-                        else:
-                            complete_response = requests.post(
-                                self._client.service_instance._href_definitions.get_attachment_href(asset_id,attachment_id)+"/complete",
-                                headers=self._client._get_headers(),
-                                params=self._client._params()
-                            )
                         if complete_response.status_code == 200:
                             print("SUCCESS")
                             return self._get_required_element_from_response(asset_details)
@@ -876,12 +861,9 @@ class Shiny(WMLResource):
     #
     #     Assets._validate_type(asset_uid, u'asset_uid', STR_TYPE, True)
     #
-    #     if not self._ICP:
-    #         response = requests.get(self._client.service_instance._href_definitions.get_data_asset_href(asset_uid), params=self._client._params(),
-    #                                 headers=self._client._get_headers())
-    #     else:
-    #         response = requests.get(self._client.service_instance._href_definitions.get_data_asset_href(asset_uid), params=self._client._params(),
-    #                                 headers=self._client._get_headers())
+    #     response = requests.get(self._client.service_instance._href_definitions.get_data_asset_href(asset_uid), params=self._client._params(),
+    #                             headers=self._client._get_headers())
+    #
     #     if response.status_code == 200:
     #         return response.json()["href"]
     #     else:
