@@ -35,7 +35,8 @@ __all__ = [
     'all_logging_disabled',
     'check_if_ts_pipeline_is_winner',
     'get_values_for_imputation_strategy',
-    'translate_imputation_string_strategy_to_enum'
+    'translate_imputation_string_strategy_to_enum',
+    'translate_estimator_string_to_enum'
 ]
 
 #  (C) Copyright IBM Corp. 2021.
@@ -1231,10 +1232,22 @@ def check_dependencies_versions(request_json: dict, wml_client, estimator_pkg: s
 
                         errored_packages.append(package)
 
-                # Note: temporary check / workaround for lightgbm on WS
                 elif package['name'] == 'lightgbm' or package['name'] == 'py-lightgbm':
-                    if installed_module_version == '3.1.1' or installed_module_version == '3.2.1':
-                        pass
+                    if package['version'] == '3.1.1':
+                        # Note: temporary check / workaround for lightgbm on WS
+                        if installed_module_version == '3.1.1' or installed_module_version == '3.2.1':
+                            pass
+                        else:
+                            errored_packages.append(package)
+                    else:
+                        lightgbm_installed_version = version.parse(installed_module_version)
+                        lightgbm_sw_spec_version = version.parse(package['version'])
+
+                        if lightgbm_installed_version.major != lightgbm_sw_spec_version.major:
+                            errored_packages.append(package)
+
+                        elif lightgbm_installed_version.minor != lightgbm_sw_spec_version.minor:
+                            errored_packages.append(package)
 
                 else:
                     if installed_module_version != package['version']:
@@ -1990,3 +2003,21 @@ def translate_imputation_string_strategy_to_enum(strategy, prediction_type):
     strategy = re.sub(r'(?<!^)(?=[A-Z])', '_', strategy).lower()
 
     return ImputationStrategy(strategy)
+
+
+def translate_estimator_string_to_enum(estimator):
+    algorithms = [ClassificationAlgorithms, RegressionAlgorithms, ForecastingAlgorithms]
+    if type(estimator) is str:
+        for alg in algorithms:
+            alg_names = [a.value for a in alg]
+
+            if estimator in alg_names:
+                return alg(estimator)
+
+            if not estimator.endswith('Estimator') and estimator + 'Estimator' in alg_names:
+                return alg(estimator + 'Estimator')
+
+            if estimator.endswith('Estimator') and estimator[:-9] in alg_names:
+                return alg(estimator[:-9])
+
+    return estimator
