@@ -25,7 +25,7 @@ from typing import Dict, List, TYPE_CHECKING
 from uuid import uuid4
 
 import pandas as pd
-import requests
+import ibm_watson_machine_learning._wrappers.requests as requests
 
 from .watson_studio import get_project
 
@@ -229,8 +229,7 @@ def save_computed_pipelines_for_ui(wml_client: 'APIClient', run_id: str) -> None
             url=f"{wml_client.service_instance._href_definitions.get_wsd_model_attachment_href()}auto_ml/{generated_path}",
             headers=wml_client._get_headers(),
             params={'space_id': wml_client.default_space_id,
-                    'flat': 'true'},
-            verify=False)
+                    'flat': 'true'})
 
         response_with_paths = wml_client.data_assets._handle_response(200, u'getting data from WMLS', response)
 
@@ -244,8 +243,7 @@ def save_computed_pipelines_for_ui(wml_client: 'APIClient', run_id: str) -> None
             response_with_file = requests.get(
                 url=f"{wml_client.service_instance._href_definitions.get_wsd_model_attachment_href()}auto_ml/{generated_path}/{pipeline_dir}/{file_name}",
                 headers=wml_client._get_headers(),
-                params={'space_id': wml_client.default_space_id},
-                verify=False)
+                params={'space_id': wml_client.default_space_id})
 
             with open(os.path.join(generated_local_path, file_name), 'wb') as f:
                 f.write(response_with_file.content)
