@@ -13,7 +13,6 @@
 #  limitations under the License.
 
 from __future__ import print_function
-from ibm_watson_machine_learning.wml_client_error import WMLClientError
 import re
 import os
 import sys
@@ -21,9 +20,15 @@ import pkg_resources
 import shutil
 import tarfile
 import logging
+import importlib.util
+
+from typing import Optional
+from subprocess import check_call
+from packaging import version
 
 import ibm_watson_machine_learning._wrappers.requests as requests
 
+from ibm_watson_machine_learning.wml_client_error import WMLClientError, CannotInstallLibrary
 
 INSTANCE_DETAILS_TYPE = u'instance_details_type'
 PIPELINE_DETAILS_TYPE = u'pipeline_details_type'
@@ -367,15 +372,6 @@ class StatusLogger:
         pass
 
 
-def version():
-    try:
-        version = pkg_resources.get_distribution("ibm-watson-machine-learning").version
-    except pkg_resources.DistributionNotFound:
-        version = u'0.0.1-local'
-
-    return version
-
-
 def get_file_from_cos(cos_credentials):
     import ibm_boto3
     from ibm_botocore.client import Config
@@ -611,4 +607,143 @@ class DisableWarningsLogger:
         logging.disable(logging.WARNING)
 
     def __exit__(self, exit_type, exit_value, exit_traceback):
-        logging.disable(logging.NOTSET)
\ No newline at end of file
+        logging.disable(logging.NOTSET)
+
+
+def is_lib_installed(lib_name: str, minimum_version: Optional[str] = None, install: Optional[bool] = False) -> bool:
+    """Check if provided library is installed on user environment. If not, tries to install it.
+
+    Parameters
+    ----------
+    lib_name: str, required
+        Library name to check.
+
+    minimum_version: str, optional
+        Minimum version of library to check. Default None, we can check if library is installed in overall.
+
+    install: bool, optional
+        Indicates to install missing or to low version library.
+
+    Returns
+    -------
+    Bool (true is library is installed, False otherwise)
+    """
+    if lib_name in sys.modules:
+        installed = True
+
+    elif importlib.util.find_spec(lib_name) is not None:
+        installed = True
+
+    else:
+        installed = False
+
+    if installed:
+        try:
+            installed_module_version = fallback_to_pip_for_version_check(lib_name=lib_name)
+
+        except Exception:
+            installed_module_version = pkg_resources.get_distribution(lib_name).version
+
+        if minimum_version is not None:
+            if version.parse(installed_module_version) < version.parse(minimum_version):
+                if install:
+                    install_library(lib_name=lib_name, version=minimum_version, strict=False)
+
+    else:
+        if install:
+            install_library(lib_name=lib_name, version=minimum_version, strict=False)
+            installed = True
+
+    return installed
+
+
+def install_library(lib_name: str, version: Optional[str] = None, strict: Optional[bool] = False) -> None:
+    """Try to install library.
+
+    Parameters
+    ----------
+    lib_name: str, required
+        Library name to install.
+
+    version: str, optional
+        Version of the library to install.
+
+    strict: bool, optional
+        Indicates if we want to install specific version or higher version if available. Default False.
+    """
+    try:
+        if version is not None:
+            check_call([sys.executable, "-m", "pip", "install", f"{lib_name}{'==' if strict else '>='}{version}"])
+
+        else:
+            check_call([sys.executable, "-m", "pip", "install", lib_name])
+
+    except Exception as e:
+        raise CannotInstallLibrary(lib_name=lib_name, reason=str(e))
+
+
+def fallback_to_pip_for_version_check(lib_name: str) -> str:
+    """Use only when you need to check package version by package name with pip."""
+    try:
+        from pip._internal.utils.misc import get_installed_distributions
+        packages = get_installed_distributions()
+        installed_module_version = None
+
+        for pkg in packages:
+            if pkg.key == lib_name:
+                installed_module_version = pkg._version
+                break
+
+        if installed_module_version is None:
+            raise pkg_resources.DistributionNotFound()
+
+        else:
+            return installed_module_version
+
+    except Exception:
+        raise pkg_resources.DistributionNotFound()
+
+
+def prepare_interaction_props_for_cos(source_params: dict, file_name: str) -> dict:
+    """If user specified properties for dataset as sheet_name, delimiter etc. we need to
+    pass them as interaction properties for Flight Service.
+
+    Parameters
+    ----------
+    source_params: dictionary, required
+        Data source parameters describe data (eg. excel_sheet, encoding etc.)
+
+    file_name: str, required
+        Name of the file to download, should consist of file extension.
+
+    Returns
+    -------
+    Dictionary with COS interaction properties for Flight Serivce.
+    """
+    interaction_properties = {}
+    file_format = None
+
+    encoding = source_params.get("encoding", None)
+
+    if ".xls" in file_name or ".xlsx" in file_name:
+        file_format = "excel"
+        interaction_properties["sheet_name"] = str(source_params.get("excel_sheet", 0))
+
+    elif '.csv' in file_name:
+        if encoding is not None:
+            interaction_properties["encoding"] = encoding
+
+        input_file_separator = source_params.get("input_file_separator", ",")
+        if input_file_separator != ",":
+            file_format = "delimited"
+            interaction_properties["field_delimiter"] = input_file_separator
+        else:
+            file_format = "csv"
+
+    elif '.parquet' in file_name or '.prq' in file_name:
+        file_format = 'parquet'
+
+    if file_format is not None:
+        interaction_properties["file_format"] = file_format
+
+    return interaction_properties
